################### AN ADAPTED LOSS FUNCTION  #######################
################ FOR CENSORED QUANTILE REGRESSION ###################
###################### LINEAR REGRESSION ############################

#   - Date:      22/03/2017
#   - Simple R Code for the proposed methodology and several competitors
#   - Parts:
#     - PART A: FUNCTIONS
#         - New estimator
#         - Wang and Wang estimator (2009)
#         - Bang and Tsiatis estimator (2002)
#     - PART B: SIMULATED DATASET

library("quantreg")

  ############# A| USED FUNCTIONS  ################
  #################################################

    # --------------------------------------------- #
    # ------------- 1. OUR ALGORITHM -------------- #
    # --------------------------------------------- #

      # New loss: MM algo 
        MMLQR.cens <- function(Y,delta,X,tau,beta,h,toler=1e-10,maxit=5000){
          
          iteration <- 0
          n.obs     <- length(Y)
          df.mat    <- cbind(1,X)
          
          # Calculation of epsilon
          tn        <- toler/n.obs
          e0        <- -tn/log(tn)
          eps       <- (e0-tn)/(1+log(e0))
          
          # Conditional Kaplan-Meier for G_C(.|X): code Wang and Wang
          Bnk.func    <- function(x0, x, h){
            xx             <- (x-x0)/h  
            xx[abs(xx)>=1] <- 1
            w              <- 15*(1-xx^2)^2/16 
            w              <- w/ifelse(sum(w)==0,1,sum(w))
            return(w)
          } 
          tauhat.func <- function(y0, x0, z, x, delta,h){
           
            n<-length(z)
            ###kernel weights#########################
            Bn = Bnk.func(x0, x, h)
            if (y0<max(z))
            {
              # sort the data z, and the delta, Bn correspondingly to the order of sorted y
              z2 = sort(z)
              Order = order(z) # so z[Order] = z2
              Bn2 = Bn[Order]
              delta2 = delta[Order]
              eta = which(delta2==1 & z2<=y0) # the index of those observations satisfying delta2==1 & z2<=y0
              Bn3 = Bn2[n:1]                  # change the order of Bn2, make the first obs of Bn2 to be the last of Bn3
              tmp = 1- Bn2 /cumsum(Bn3)[n:1]  
              out = 1-prod(tmp[eta], na.rm=T) # na.rm=T, as some of those tmp=NA as the denom =0
            } 
            else out<-1 
            return(out)
          }
          Beran.fun   <- function(a,h){ # rewritten to use apply on rows of matrix of (beta X,X) in MM afterwards
            tauhat.func(a[1],a[2], Y, X, 1-delta,h)
          }
          
          # Initialization of condition for break
          cond <- T
          while(cond)
          { 
            beta.prev <- beta
            
            r.vec     <- Y-df.mat%*%as.matrix(beta)
            A.entries <- c(1/(eps+abs(r.vec)))/2
            A.mat     <- diag(A.entries)
            
            B.mat     <- as.matrix(rep(tau-.5,n.obs))
            G.mat     <- as.matrix((1-tau)*apply(cbind(df.mat%*%as.matrix(beta.prev),X),1,FUN="Beran.fun",h=h))
            
            beta      <- solve(t(df.mat)%*%A.mat%*%df.mat)%*%(t(df.mat)%*%(A.mat%*%as.matrix(Y)+B.mat+G.mat))
            cond      <- max(abs(beta-beta.prev)) > toler
            iteration <- iteration + 1
            
            if(iteration > maxit){warning("WARNING: Algorithm did not converge"); break} 
          }
          return(list("beta"=c(beta),"IterN"=iteration))
        }
 
    # --------------------------------------------- #
    # -------------- 2. LITERATURE ---------------- #
    # --------------------------------------------- #
      
      # Wang and Wang: code available online
        Bnk.func    <- function(x0, x, h){
  # the kernel weight function Bnk(x0, x), where x0 is a scalar, and x is a vector
  # returns a vector
  # h is the bandwidth
  xx<-(x-x0)/h  
  xx[abs(xx)>=1]<-1
  w<-15*(1-xx^2)^2/16  #biquadratic kernel 
  w<-w/sum(w)
  return(w)
} 
        tauhat.func <- function(y0, x0, z, x, delta,h){
  # tau0(y0, x0) = F(T<y0|x0); so y0 is the C_i, and x0 is the xi in the paper
  # z is observed vector of response variable
  # x is the observed covariate
  # delta is the censoring indicator function
  # h is the bandwidth
  n<-length(z)
  ###kernel weights#########################
  Bn = Bnk.func(x0, x, h)
  if (y0<max(z))
  {
    # sort the data z, and the delta, Bn correspondingly to the order of sorted y
    z2 = sort(z)
    Order = order(z) # so z[Order] = z2
    Bn2 = Bn[Order]
    delta2 = delta[Order]
    eta = which(delta2==1 & z2<=y0) # the index of those observations satisfying delta2==1 & z2<=y0
    Bn3 = Bn2[n:1]  # change the order of Bn2, make the first obs of Bn2 to be the last of Bn3
    tmp = 1- Bn2 /cumsum(Bn3)[n:1]  
    out = 1-prod(tmp[eta], na.rm=T) # na.rm=T, as some of those tmp=NA as the denom =0
  } 
  else out<-1 
  return(out)
}
        WW.cens     <- function(y, x, delta, tau, h){
  # x is one dimensional, not including the intercept yet
  # y is the observed survival time = min(T, C)
  # delta is the censoring indicator function with 1 standing for uncensored, and 0 censored
  # tau is the quantile level of interest
  # h is the handwidth used in calculating tauhat for each censored observation
  n = length(y)
  ind = which(delta==0)
  w = rep(1, n)  # the weight
  if(length(ind)>=1)
  {
    for(i in 1:length(ind))
    {
      x0 = x[ind[i]]
      y0 = y[ind[i]]
      tau.star = tauhat.func(y0,x0, y, x, delta,h=h)
      if (tau>tau.star) w[ind[i]] = (tau-tau.star)/(1-tau.star)
    }
    # pseudo observations
    ind2 = which(w!=1)
    y.pse = rep(max(y)+100, length(ind2))
    x.pse = x[ind2]
    yy = c(y, y.pse)
    xx = c(x, x.pse)
    ww = c(w, 1-w[ind2])
  }
  else
  {
    yy=y; xx=x; ww=w
  }
  rq1 = rq(yy~xx, weights=ww, tau=tau)
  result<- rq1$coeff
  return(list(coeff=result,weights=w))    
}
        
      # Bang and Tsiatis
        BT.fun <- function(Y,delta,X,tau){
          require(quantreg)
          # Kaplan-Meier estimator (strict inequality)
          ind.str        <- function(x,a,b)  ifelse(x >= a & x < b, 1,0)
          fonc.G         <- function(ordre,Y,delta){
            n<-length(Y)
            indice<-order(Y)
            prod<-1
            if(ordre>1){ordre<-ordre-1}
            for(j in 1:ordre){prod<-prod*((n-j)/(n-j+1))^(1-delta[indice[j]])}
            return(1-prod)
          }
          poids.KM       <- function(ordre,Y,delta){ # Attention divis? par (n+1)!!
            pds<-delta[ordre]/((length(Y)+1)*(1-fonc.G(ordre,Y,delta)))
            return(pds)
          }
          KM.str.fun     <- function(t,Y,delta){ #Attention K-M de fonction de distribution!!!! INEGALITE STRICTE DANS INDICATRICE
            indice<-order(Y)
            Y1<-sort(Y)
            delta1<-c()
            for(j in 1:length(Y)){delta1[j]<-delta[indice[j]]}
            m<-length(Y);sum<-0
            for(i in 1:m){sum<-sum+poids.KM(i,Y1,delta1)*ind.str(Y1[i],-Inf,t)}
            return(sum)
          }
          rq(Y~X,tau,weights=delta/(1-KM.str.fun(Y,Y,1-delta)))$coef
        }
        
  ############# B| SIMULATED DATA ################
  ################################################

    # Generate Data from DGP 1
      DataGen.DGP1 <- function(n,tau=.5,prop.cens=.4){
          b0  <- 3
          b1  <- 5 
          X   <- runif(n)
          eta <- rnorm(n)
          eps <- eta - qnorm(tau)
          
          T <- b0+b1*X+eps

          max.C     <- 14
          C         <- runif(n,min=0,max=max.C)
          Y         <- pmin(T,C)
          delta     <- T <= C
          
          result     <- matrix(nrow= n, ncol=5)
          result[,1] <- T
          result[,2] <- Y
          result[,3] <- delta
          result[,4] <- X
          result[,5] <- C
          
          return(result)
      }
    
    # Setting
      tau       <- .5
      prop.cens <- .4
      n.obs     <- 200
      beta      <- c(3,5)
      
    # Simulate data
      set.seed(2015)
      
      data.sim  <- DataGen.DGP1(n.obs)
      T.sim     <- data.sim[,1]
      Y.sim     <- data.sim[,2]
      delta.sim <- data.sim[,3]
      X.sim     <- data.sim[,4]
      
    # Estimation
      
        # Fully observed data
          (m.est.full <- rq(T.sim~X.sim,tau)$coef)
        
        # Censored data 
          
           # 1/ Wang and Wang
              (m.est.WW <- WW.cens(Y.sim, X.sim, delta.sim, tau=tau, h=.05)$coeff)          
          
           # 2/ Bang and Tsiatis
              (m.est.BT <- BT.fun(Y.sim, delta.sim, X.sim, tau=tau))

           # 3/ New estimator 
              (m.est.NEW <- MMLQR.cens(Y.sim,delta.sim,X.sim,tau,m.est.BT+1e-2,h=.05))

 